@@ -1,5 +1,5 @@
 // ePDFView - A lightweight PDF Viewer.
-// Copyright (C) 2006 Emma's Software.
+// Copyright (C) 2006, 2007, 2009 Emma's Software.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 #include <config.h>
 #include <string.h>
+#include <gdk/gdk.h>
 #include "epdfview.h"
 
 using namespace ePDFView;
@@ -28,6 +29,7 @@ static const gint BYTES_PER_PIXEL = 3;
 ///
 DocumentPage::DocumentPage ()
 {
+    m_Selection = NULL;
     m_Data = NULL;
     m_HasSelection = FALSE;
     m_Height = 0;
@@ -49,6 +51,9 @@ DocumentPage::~DocumentPage ()
         delete link;
     }
     g_list_free (m_LinkList);
+
+    if(m_Selection)
+        gdk_region_destroy(m_Selection);
 }
 
 ///
@@ -79,6 +84,12 @@ DocumentPage::clearSelection ()
         invertArea (m_SelectionX1, m_SelectionY1, m_SelectionX2, m_SelectionY2);
         m_HasSelection = FALSE;
     }
+
+    if(NULL != m_Selection){
+        invertRegion(m_Selection);
+        gdk_region_destroy(m_Selection);
+        m_Selection = NULL;
+    }
 }
 
 ///
@@ -195,6 +206,20 @@ DocumentPage::invertArea (gint x1, gint y1, gint x2, gint y2)
     }
 }
 
+void
+DocumentPage::invertRegion (GdkRegion* region)
+{
+    int count;
+    GdkRectangle *rectangles;
+    gdk_region_get_rectangles(region, &rectangles, &count);
+    while(count--){
+        GdkRectangle r = rectangles[count];
+        invertArea(r.x, r.y, r.x + r.width, r.y + r.height);
+    }
+    g_free(rectangles);
+}
+
+
 ///
 /// @brief Allocates the memory for a new page.
 ///
@@ -253,3 +278,13 @@ DocumentPage::setSelection (DocumentRectangle &selection, gdouble scale)
     
     m_HasSelection = TRUE;
 }
+
+void
+DocumentPage::setSelection (GdkRegion *region)
+{
+    clearSelection ();
+    
+    invertRegion (region);
+    
+    m_Selection = gdk_region_copy(region);
+}
