@@ -1,5 +1,5 @@
 // ePDFView - A lightweight PDF Viewer.
-// Copyright (C) 2006 Emma's Software.
+// Copyright (C) 2006, 2007, 2009 Emma's Software.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 #if !defined (__DOCUMENT_PAGE_H__)
 #define __DOCUMENT_PAGE_H__
 
+typedef struct _GdkRegion GdkRegion;
+
 namespace ePDFView
 {
     ///
@@ -41,6 +43,7 @@ namespace ePDFView
             gint getWidth (void);
             gboolean newPage (gint width, gint height);
             void setSelection (DocumentRectangle &selection, gdouble scale);
+            void setSelection (GdkRegion *region);
 
         protected:
             /// The page's image.
@@ -61,7 +64,10 @@ namespace ePDFView
             gint m_Width;
             /// The list of links from the page.
             GList *m_LinkList;
-
+            /// Selection region
+            GdkRegion *m_Selection;
+            
+            void invertRegion (GdkRegion*);
             void invertArea (gint x1, gint y1, gint x2, gint y2);
     };
 }
