@@ -1,5 +1,5 @@
 // ePDFView - A lightweight PDF Viewer.
-// Copyright (C) 2006 Emma's Software.
+// Copyright (C) 2006, 2007, 2009 Emma's Software.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,12 +21,14 @@
 /// This is the definition of the GQuark used for Document's errors.
 #define EPDFVIEW_DOCUMENT_ERROR     ePDFView::IDocument::getErrorQuark ()
 
+typedef struct _GdkRegion GdkRegion;
+
 namespace ePDFView 
 {
     // Forward declarations.
     class DocumentIndex;
     class IDocumentObserver;
-    class DocumentPage;
+    class DocumentPage; 
 
     ///
     /// @brief Defines the possible errors loading a document.
@@ -263,6 +265,9 @@ namespace ePDFView
             virtual gboolean saveFile (const gchar *filename,
                                        GError **error) = 0;
 
+            virtual GdkRegion* getTextRegion (DocumentRectangle *rect) = 0;
+            virtual void setTextSelection (DocumentRectangle *rect) = 0;
+
             void attach (const IDocumentObserver *observer);
             void detach (const IDocumentObserver *observer);
 
