@@ -1,5 +1,5 @@
 // ePDFView - A lightweight PDF Viewer.
-// Copyright (C) 2006 Emma's Software.
+// Copyright (C) 2006, 2007, 2009 Emma's Software.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -150,6 +150,16 @@ namespace ePDFView
             ///
             virtual void notifySaveError (const GError *error) { }
 
+            ///
+            /// @brief Someone select text in document.
+            ///
+            /// This function called when someone select some text
+            /// in document.
+            ///
+            /// @param text Selected text.
+            ///
+            virtual void notifyTextSelected (const gchar* text) { }
+
         protected:
             ///
             /// @brief Constructs a new IDocumentObserver object.
