@@ -1,5 +1,5 @@
 // ePDFView - A lightweight PDF Viewer.
-// Copyright (C) 2006 Emma's Software.
+// Copyright (C) 2006, 2007, 2009 Emma's Software.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -576,6 +576,10 @@ namespace ePDFView
             ///
             virtual void setOutline (DocumentOutline *outline) = 0;
 
+            virtual void copyTextToClibboard(const gchar* text) = 0;
+            virtual void activePageModeScroll (gboolean active) = 0;
+            virtual void activePageModeText (gboolean active) = 0;
+
         protected:
             ///
             /// @brief Constructs a new main view.
