@@ -1,5 +1,5 @@
 // ePDFView - A lightweight PDF Viewer.
-// Copyright (C) 2006 Emma's Software.
+// Copyright (C) 2006, 2007, 2009 Emma's Software.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -695,6 +695,12 @@ MainPter::zoomFitActivated (gboolean active)
     }
 }
 
+void
+MainPter::setPageMode (PagePterMode mode)
+{
+    m_PagePter->setMode(mode);
+}
+
 ///
 /// @brief The "Zoom In" was activated.
 ///
@@ -995,3 +1001,9 @@ MainPter::waitForFileSaved ()
     G_UNLOCK (fileSaved);
 }
 #endif // DEBUG
+
+void 
+MainPter::notifyTextSelected (const gchar* text)
+{
+    getView ().copyTextToClibboard(text);
+}
