@@ -1,5 +1,5 @@
 // ePDFView - A lightweight PDF Viewer.
-// Copyright (C) 2006 Emma's Software.
+// Copyright (C) 2006, 2007, 2009 Emma's Software.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -191,7 +191,7 @@ PDFDocument::isLoaded ()
 /// @param filename The name of the file name to open. It must be an absolute
 ///                 path.
 /// @param password The password to use to open @a filename.
-/// @param error Location to store the error occurring or NULL to ignore 
+/// @param error Location to store the error occurring or NULL to ignore
 ///              errors.
 ///
 /// @return TRUE if the file could be opened, FALSE otherwise.
@@ -257,7 +257,7 @@ PDFDocument::loadFile (const gchar *filename, const gchar *password,
         else
         {
             // OK, the glib's wrapper don't pass the error code directly
-            // from Poppler. Instead returns G_FILE_ERROR_FAILED and a 
+            // from Poppler. Instead returns G_FILE_ERROR_FAILED and a
             // non translated string.
             // Maybe I'm wrong (very probable) but that's a wrong way.
             // So I'm reading the error code from the error string...
@@ -412,7 +412,7 @@ PDFDocument::setLinks (DocumentPage *renderedPage, PopplerPage *popplerPage)
     poppler_page_get_size (popplerPage, NULL, &pageHeight);
     // We'll already calculate the positions scaled.
     gdouble scale = getZoom ();
-    GList *pageLinks = poppler_page_get_link_mapping (popplerPage); 
+    GList *pageLinks = poppler_page_get_link_mapping (popplerPage);
     for (GList *pageLink = g_list_first (pageLinks) ;
          NULL != pageLink ;
          pageLink = g_list_next (pageLink) )
@@ -440,7 +440,7 @@ PDFDocument::setLinks (DocumentPage *renderedPage, PopplerPage *popplerPage)
 ///                     poppler_index_iter_new().
 ///
 void
-PDFDocument::setOutline (DocumentOutline *outline, 
+PDFDocument::setOutline (DocumentOutline *outline,
                          PopplerIndexIter *childrenList)
 {
     if ( NULL != childrenList )
@@ -474,11 +474,11 @@ PDFDocument::setOutline (DocumentOutline *outline,
 #endif // HAVE_POPPLER_0_5_2
 
                     outline->addChild (child);
-                PopplerIndexIter *childIter = 
+                PopplerIndexIter *childIter =
                     poppler_index_iter_get_child (childrenList);
                 setOutline (child, childIter);
             }
-        } 
+        }
         while ( poppler_index_iter_next (childrenList) );
 
         poppler_index_iter_free (childrenList);
@@ -490,7 +490,7 @@ PDFDocument::setOutline (DocumentOutline *outline,
 ///
 /// Retrieves the width and height of a document's page before to scale, but
 /// after rotation.
-/// 
+///
 /// @param pageNum The page to get its size.
 /// @param width The output pointer to save the page's width.
 /// @param height The output pointer to save the page's height.
@@ -656,6 +656,107 @@ PDFDocument::saveFile (const gchar *filename, GError **error)
     return result;
 }
 
+static void
+repairEmpty(PopplerRectangle& rect)
+{
+    //rect must have be least 1x1
+    if(rect.y1 == rect.y2)
+        rect.y2++;
+    if(rect.x1 == rect.x2)
+        rect.x2++;
+}
+
+void
+PDFDocument::setTextSelection (DocumentRectangle *rect)
+{
+    g_assert(rect);
+
+    PopplerPage *page = poppler_document_get_page (m_Document, getCurrentPageNum()-1);
+    if(!page)
+        return;
+
+    gdouble pageWidth, pageHeight;
+    poppler_page_get_size(page, &pageWidth, &pageHeight);
+
+    //for get text we must exchange y coordinate, don't ask me where logic here.
+    PopplerRectangle textRect = { rect->getX1() / getZoom(),
+                                  (pageHeight - rect->getY2() / getZoom()),
+                                  rect->getX2() * getZoom() / getZoom(),
+                                  (pageHeight - rect->getY1() / getZoom())};
+    repairEmpty(textRect);
+
+#if defined (HAVE_POPPLER_0_6_0)
+    gchar *text = poppler_page_get_text(page, POPPLER_SELECTION_GLYPH,
+            &textRect);
+#else // !HAVE_POPPLER_0_6_0
+    gchar *text = poppler_page_get_text(page, &textRect);
+#endif // HAVE_POPPLER_0_6_0
+    if(!text)
+        goto cleanup;
+
+    for ( GList *obs = g_list_first (m_Observers) ;
+          NULL != obs ;
+          obs = g_list_next (obs) )
+        {
+            IDocumentObserver *observer = (IDocumentObserver*)obs->data;
+            observer->notifyTextSelected(text);
+        }
+
+ cleanup:
+    if(page)
+        g_object_unref(page);
+    if(text)
+        g_free(text);
+}
+
+GdkRegion*
+PDFDocument::getTextRegion (DocumentRectangle *r)
+{
+    GdkRegion *res = NULL;
+    PopplerPage *page = poppler_document_get_page (m_Document, getCurrentPageNum()-1);
+    if(!page)
+        return NULL;
+
+    //calulate rect
+    PopplerRectangle rect;
+    rect.x1 = r->getX1() / getZoom();
+    rect.y1 = r->getY1() / getZoom();
+    rect.x2 = r->getX2() / getZoom();
+    rect.y2 = r->getY2() / getZoom();
+    repairEmpty(rect);
+
+    //calc selection size
+#if defined (HAVE_POPPLER_0_8_0)
+    GList *selections = poppler_page_get_selection_region(page, getZoom(),
+             POPPLER_SELECTION_GLYPH, &rect);
+    res = gdk_region_new();
+    for (GList *selection = g_list_first (selections) ;
+         NULL != selection ; selection = g_list_next (selection)) {
+        PopplerRectangle *rectangle = (PopplerRectangle *)selection->data;
+        GdkRectangle rect;
+
+        rect.x = (gint)rectangle->x1;
+        rect.y = (gint)rectangle->y1;
+        rect.width  = (gint) (rectangle->x2 - rectangle->x1);
+        rect.height = (gint) (rectangle->y2 - rectangle->y1);
+
+        gdk_region_union_with_rect (res, &rect);
+    }
+    poppler_page_selection_region_free (selections);
+#elif defined (HAVE_POPPLER_0_6_0)
+    res = poppler_page_get_selection_region(page, getZoom(),
+            POPPLER_SELECTION_GLYPH, &rect);
+#else // !HAVE_POPPLER_0_6_0
+    res = poppler_page_get_selection_region(page, getZoom(), &rect);
+#endif // HAVE_POPPLER_0_6_0
+
+    //free some local data
+    g_object_unref(page);
+
+    return res;
+}
+
+
 ///
 /// @brief Gets the document's page layout from Poppler's page layout.
 ///
