@@ -1,5 +1,5 @@
 // ePDFView - A lightweight PDF Viewer.
-// Copyright (C) 2006 Emma's Software.
+// Copyright (C) 2006, 2007, 2009 Emma's Software.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -184,6 +184,9 @@ PageView::setCursor (PageCursor cursorType)
         GdkCursor *cursor = NULL;
         switch (cursorType)
         {
+            case PAGE_VIEW_CURSOR_SELECT_TEXT:
+                cursor = gdk_cursor_new (GDK_XTERM);
+                break;
             case PAGE_VIEW_CURSOR_LINK:
                 cursor = gdk_cursor_new (GDK_HAND2);
                 break;
@@ -524,6 +527,13 @@ page_view_scrolled_cb (GtkWidget *widget, GdkEventScroll *event, gpointer data)
 {
     g_assert ( NULL != data && "The data parameter is NULL.");
 
+    // don't scroll when the CRTL-Button is down, because then the page should
+    // actually be zoomed and not scrolled. Zooming is handelt by the MainView
+    // class.
+    if ( event->state & GDK_CONTROL_MASK )
+    {
+        return FALSE;
+    }
     PagePter *pter = (PagePter *)data;
     GtkAdjustment *adjustment = 
         gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (widget));
@@ -553,11 +563,15 @@ page_view_keypress_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
 
     GtkScrollType direction;
     gboolean horizontal = FALSE;
+    gboolean returnValue = TRUE;
     PagePter *pter = (PagePter *)data;
 
-    GtkAdjustment *adjustment = 
+    GtkAdjustment *hadjustment = 
+        gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (widget));
+    gdouble hposition = gtk_adjustment_get_value (hadjustment);
+    GtkAdjustment *vadjustment = 
         gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (widget));
-    gdouble position = gtk_adjustment_get_value (adjustment);
+    gdouble vposition = gtk_adjustment_get_value (vadjustment);
 
     if ( event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK) )
     {
@@ -568,51 +582,96 @@ page_view_keypress_cb(GtkWidget *widget, GdkEventKey *event, gpointer data)
     {
         case GDK_Left:
         case GDK_KP_Left:
+        case GDK_h:
+            if ( hposition == hadjustment->lower )
+            {
+                pter->scrollToPreviousPage ();
+                return TRUE;
+            }
             direction = GTK_SCROLL_STEP_LEFT;
             horizontal = TRUE;
             break;
 
         case GDK_Right:
         case GDK_KP_Right:
+        case GDK_l:
+            if ( hposition == ( hadjustment->upper - hadjustment->page_size) )
+            {
+                pter->scrollToNextPage ();
+                return TRUE;
+            }
             horizontal = TRUE;
             direction = GTK_SCROLL_STEP_RIGHT;
             break;
 
         case GDK_Up:
         case GDK_KP_Up:
+        case GDK_k:
+            if ( vposition == vadjustment->lower )
+            {
+                pter->scrollToPreviousPage ();
+                return TRUE;
+            }
             direction = GTK_SCROLL_STEP_UP;
             break;
 
         case GDK_Down:
         case GDK_KP_Down:
+        case GDK_j:
+            if ( vposition == ( vadjustment->upper - vadjustment->page_size) )
+            {
+                pter->scrollToNextPage ();
+                return TRUE;
+            }
             direction = GTK_SCROLL_STEP_DOWN;
             break;
 
         case GDK_Page_Up:
         case GDK_KP_Page_Up:
-            if ( position == adjustment->lower )
+            if ( vposition == vadjustment->lower )
             {
                 pter->scrollToPreviousPage ();
                 return TRUE;
             }
-            direction = GTK_SCROLL_START;
+            direction = GTK_SCROLL_PAGE_UP;
             break;
 
+        case GDK_space:
+        case GDK_KP_Space:
         case GDK_Page_Down:
         case GDK_KP_Page_Down:
-            if ( position == ( adjustment->upper - adjustment->page_size) )
+            if ( vposition == ( vadjustment->upper - vadjustment->page_size) )
             {
                 pter->scrollToNextPage ();
                 return TRUE;
             }
+            direction = GTK_SCROLL_PAGE_DOWN;
+            break;
+
+       case GDK_Home:
+       case GDK_KP_Home:
+            direction = GTK_SCROLL_START;
+            break;
+
+       case GDK_End:
+       case GDK_KP_End:
             direction = GTK_SCROLL_END;
             break;
 
-        default:
+       case GDK_Return:
+       case GDK_KP_Enter:
+            pter->scrollToNextPage ();
+            direction = GTK_SCROLL_START;
+
+       case GDK_BackSpace:
+            pter->scrollToNextPage ();
+            direction = GTK_SCROLL_START;
+
+       default:
             return FALSE;
     }
 
     g_signal_emit_by_name(G_OBJECT(widget), "scroll-child",
-                          direction, horizontal);
-    return TRUE;
+                          direction, horizontal, &returnValue);
+    return returnValue;
 }
