@@ -1,5 +1,5 @@
 // ePDFView - A lightweight PDF Viewer.
-// Copyright (C) 2006 Emma's Software.
+// Copyright (C) 2006, 2007, 2009 Emma's Software.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -729,6 +729,12 @@ PrintView::createLayoutListModel ()
                         printOptionLabelColumn, _("4 pages in 1"),
                         printOptionValueColumn, PRINT_PAGE_LAYOUT_4IN1,
                         -1);
+
+    gtk_list_store_append (m_Layout, &iter);
+    gtk_list_store_set (m_Layout, &iter,
+                        printOptionLabelColumn, _("6 pages in 1"),
+                        printOptionValueColumn, PRINT_PAGE_LAYOUT_6IN1,
+                        -1);
 }
 
 void
